/**************************************************************************//**
* @file     can.c
* @version  V1.00
* $Revision: 1 $
* $Date: 15/06/11 2:44p $
* @brief    NUC970 CAN driver source file
*
* @note
* Copyright (C) 2015 Nuvoton Technology Corp. All rights reserved.
*****************************************************************************/
#include <rthw.h>
#include <rtthread.h>
#include <rtdevice.h>
#include "nuc97x.h"
#include "nuc97x_can.h"
#include <ipc/completion.h>
#include <drivers/can.h>


/** @addtogroup NUC970_Device_Driver NUC970 Device Driver
  @{
*/

/** @addtogroup NUC970_CAN_Driver CAN Driver
  @{
*/

/** @addtogroup NUC970_CAN_EXPORTED_CONSTANTS CAN Exported Constants
  @{
*/

/*@}*/ /* end of group NUC970_CAN_EXPORTED_CONSTANTS */

#include <stdio.h>


/// @cond HIDDEN_SYMBOLS


//#define CAN_DEBUG
#ifndef CAN_DEBUG
#define CAN_DBG(...)  rt_kprintf("CAN]:"),rt_kprintf(__VA_ARGS__)
#else
#define CAN_DBG(...)
#endif /* CAN_DEBUG */

/**
  * @brief    Enter initialization mode
  * @param[in]    uCAN    CAN channel.
  * @return   None
  * @details  This function is used to set CAN to enter initialization mode and enable access bit timing
  *           register. After bit timing configuration ready, user must call CAN_LeaveInitMode()
  *           to leave initialization mode and lock bit timing register to let new configuration
  *           take effect.
  */
void CAN_EnterInitMode(struct c_can_priv *  can_priv)
{
    can_priv->write_reg(can_priv,REG_CON, can_priv->read_reg(can_priv,REG_CON)|CAN_CON_INIT_Msk);//into initalization mode
    can_priv->write_reg(can_priv,REG_CON, can_priv->read_reg(can_priv,REG_CON)|CAN_CON_CCE_Msk); //enable modify CAN_BTIME & CAN_BRP register
}


/**
  * @brief    Leave initialization mode
  * @param[in]    uCAN    CAN channel.
  * @return   None
  * @details  This function is used to set CAN to leave initialization mode to let
  *           bit timing configuration take effect after configuration ready.
  */
void CAN_LeaveInitMode(struct c_can_priv *  can_priv)
{
    UINT32  uRegVal;
    uRegVal = can_priv->read_reg (can_priv, REG_CON);
    uRegVal &= (~(CAN_CON_INIT_Msk | CAN_CON_CCE_Msk));//disable modify CAN_BTIME & CAN_BRP register
    can_priv->write_reg (can_priv,REG_CON, uRegVal);

    while(can_priv->read_reg (can_priv ,(REG_CON) & CAN_CON_INIT_Msk));//waiting for init bit  is reset;
}






/**
  * @brief      get Tx request pending status
  * @param[in]	u8MsgObj    Specifies the Message object number, from 0 to 31.
  * @retval     non-zero    The corresponding message object has a new data bit is set.
  * @retval     0           No message object has new data.
  * @details    This function is used to get the request pending status of a transmitted message.
  */

rt_err_t CAN_IsTxRqstPending(struct c_can_priv *can_priv,rt_uint8_t u8MsgObj)
{
  	if (u8MsgObj < 16 ? can_priv->read_reg(can_priv,REG_TXREQ1) & (1 << u8MsgObj) : can_priv->read_reg(can_priv,REG_TXREQ2) & (1 << (u8MsgObj-16)))
  	{
  		rt_base_t level= rt_hw_interrupt_disable();
  		CAN_DBG(" %d the  %s   %d     channel is request pending\r\n",__LINE__, can_priv->name, u8MsgObj);
  		rt_hw_interrupt_enable( level);
  		return RT_EOK;
  	}
  	else
  	{
  		CAN_DBG("%d the  %s  %d  channel is not request   pending\r\n",__LINE__, can_priv->name,u8MsgObj);
  		return RT_ERROR;
  	}
}


/**
  * @brief      Get message object interrupt status
  * @param[in]	u8MsgObj    Specifies the Message object number, from 0 to 31.
  * @retval     non-zero    The corresponding message object has a new data bit is set.
  * @retval     0           No message object has new data.
  * @details    This function is used to get the interrupt status of a message object
  */

rt_err_t CAN_IsIntPending(struct c_can_priv *can_priv,rt_uint8_t u8MsgObj)
{
  	if (u8MsgObj < 16 ? can_priv->read_reg(can_priv,REG_IPND1) & (1 << u8MsgObj) : can_priv->read_reg(can_priv,REG_IPND2) & (1 << (u8MsgObj-16)))
  	{
  		CAN_DBG("%d the  %s  %d   channel is interrupt pending\r\n",__LINE__, can_priv->name, u8MsgObj);
  		return RT_EOK;
  	}
  	else
  	{
  		CAN_DBG("  %d the %s  %d   channel is   not interrupt pending\r\n",__LINE__, can_priv->name, u8MsgObj);
  		return RT_ERROR;
  	}
}


/**
  * @brief      Test the validity of a message object
  * @param[in]	u8MsgObj    Specifies the Message object number, from 0 to 31.
  * @retval     non-zero    The corresponding message object has a new data bit is set.
  * @retval     0           No message object has new data.
  * @details    This function is used to test the validity of a message object (ready to use).
  */

rt_err_t CAN_IsObjectValid(struct c_can_priv *can_priv,rt_uint8_t u8MsgObj)
{
  	if (u8MsgObj < 16 ? can_priv->read_reg(can_priv,REG_MVLD1) & (1 << u8MsgObj) : can_priv->read_reg(can_priv,REG_MVLD2) & (1 << (u8MsgObj-16)))
  		{
  		CAN_DBG(" %d  the %s %d  channel is validity\r\n",__LINE__, can_priv->name, u8MsgObj);
  		return RT_EOK;
  		}
  	else
  		{
  		CAN_DBG("%d  the %s %d   channel is not validity\r\n",__LINE__, can_priv->name, u8MsgObj);
  		return RT_ERROR;
  		}
}






/**
  * @brief    Wait message into message buffer in basic mode.
  * @param[in]    uCAN    CAN channel.
  * @return   None
  * @details  This function is used to wait message into message buffer in basic mode. Please notice the
  *           function is polling NEWDAT bit of MCON register by while loop and it is used in basic mode.
  */
void CAN_WaitMsg(struct c_can_priv * can_priv)
{

    can_priv->write_reg (can_priv,(REG_STATUS ), 0x0);

    while(1)
    {
        if(can_priv->read_reg (can_priv,REG_IF2_MCON ) & CAN_IF_MCON_NEWDAT_Msk) //The Message Handler or the application software has written new data into the data portion of this Message Object.
        {
        	CAN_DBG("%d the %s  New Data IN\r\n",__LINE__, can_priv->name);
            break;
        }

        if(can_priv->read_reg (can_priv ,REG_STATUS ) & CAN_STATUS_RXOK_Msk) //A message has been successfully received since this bit was last reset by the CPU (independent of the result of acceptance filtering).
        	CAN_DBG("the   %s Rx OK\r\n",__LINE__, can_priv->name);

        if(can_priv->read_reg (can_priv,REG_STATUS ) & CAN_STATUS_LEC_Msk)// check error status;
        	CAN_DBG(" %d the  %s  Error \r\n",__LINE__, can_priv->name );
    }

}

/**
  * @brief    Get current bit rate
  * @param[in]    uCAN    CAN channel.
  * @return   Current Bit-Rate (kilo bit per second)
  * @details  Return current CAN bit rate according to the user bit-timing parameter settings
  */
rt_uint32_t CAN_GetCANBitRate(struct c_can_priv * can_priv)
{
    UINT32 uRegAddress;

    UINT8 u8Tseg1,u8Tseg2;
    UINT32 u32Bpr, u32Value;

    u32Value = sysGetClock(SYS_PCLK)*1000000;

    u8Tseg1 = (can_priv->read_reg (can_priv,REG_BTIME) & CAN_BTIME_TSEG1_Msk) >> CAN_BTIME_TSEG1_Pos;
    u8Tseg2 = (can_priv->read_reg (can_priv,REG_BTIME) & CAN_BTIME_TSEG2_Msk) >> CAN_BTIME_TSEG2_Pos;
    u32Bpr  = (can_priv->read_reg (can_priv,REG_BTIME) & CAN_BTIME_BRP_Msk) | (can_priv->read_reg (can_priv,REG_BRPE ) << 6);

    return (u32Value/(u32Bpr+1)/(u8Tseg1 + u8Tseg2 + 3));
}

/**
  * @brief    Switch the CAN into test mode.
  * @param[in]    uCAN    CAN channel.
  * @param[in]    u8TestMask  Specifies the configuration in test modes
  *                             CAN_TEST_BASIC_Msk   : Enable basic mode of test mode
  *                             CAN_TESTR_SILENT_Msk  : Enable silent mode of test mode
  *                             CAN_TESTR_LBACK_Msk   : Enable Loop Back Mode of test mode
  *                             CAN_TESTR_TX0_Msk/CAN_TESTR_TX1_Msk: Control CAN_TX pin bit field
  * @return   None
  * @details  Switch the CAN into test mode. There are four test mode (BASIC/SILENT/LOOPBACK/
  *           LOOPBACK combined SILENT/CONTROL_TX_PIN)could be selected. After setting test mode,user
  *           must call CAN_LeaveInitMode() to let the setting take effect.
  */
void CAN_EnterTestMode(struct c_can_priv * can_priv, rt_uint8_t u8TestMask)
{
    UINT32 uRegVal;

    uRegVal = can_priv->read_reg (can_priv,REG_CON );
    uRegVal |= CAN_CON_TEST_Msk;
    can_priv->write_reg (can_priv,(REG_CON ), uRegVal);

    can_priv->write_reg (can_priv,(REG_TEST ), u8TestMask);
}




/**
  * @brief    Leave the test mode
  * @param[in]    uCAN    CAN channel.
  * @return   None
  * @details  This function is used to Leave the test mode (switch into normal mode).
  */
void CAN_LeaveTestMode(struct c_can_priv *can_priv)
{
    UINT32 uRegVal;

    uRegVal = can_priv->read_reg (can_priv,REG_CON );
    uRegVal |= CAN_CON_TEST_Msk;                //into  test mode  
    can_priv->write_reg (can_priv,(REG_CON ), uRegVal);

    uRegVal = can_priv->read_reg (can_priv,REG_TEST );
    uRegVal &= ~CAN_CON_TEST_Msk;              //leave test mode ... 
    can_priv->write_reg (can_priv,(REG_TEST ), uRegVal);

    uRegVal = can_priv->read_reg (can_priv,REG_CON );
    uRegVal &= ~CAN_CON_TEST_Msk;               // leave test mode  
    can_priv->write_reg (can_priv,(REG_CON ), uRegVal);
}

/**
  * @brief    Get the waiting status of a received message.
  * @param[in]    uCAN    CAN channel.
  * @param[in]    u8MsgObj    Specifies the Message object number, from 0 to 31.
  * @retval   non-zero    The corresponding message object has a new data bit is set.
  * @retval   0           No message object has new data.
  * @details  This function is used to get the waiting status of a received message.
  */
rt_uint32_t CAN_IsNewDataReceived(struct c_can_priv *can_priv, rt_uint8_t u8MsgObj)
{

    rt_uint16_t i = (u8MsgObj < 16 ? (can_priv->read_reg (can_priv,REG_NDAT1 ) & (1 << u8MsgObj)) :(can_priv->read_reg (can_priv,REG_NDAT2 ) & (1 << (u8MsgObj-16))));
     if(i!=0)
    {
	CAN_DBG("%d  the new data received  %s   %d     \r\n",__LINE__, can_priv->name, u8MsgObj);

    return i;
    }
    else
    {
    	CAN_DBG(" %d  no new data received   %s   %d     \r\n",__LINE__, can_priv->name, u8MsgObj);
    	return i;
    }
}


/**
  * @brief    Send CAN message in BASIC mode of test mode
  * @param[in]    uCAN    CAN channel.
  * @param[in]    pCanMsg     Pointer to the message structure containing data to transmit.
  * @return   TRUE:  Transmission OK
  *           FALSE: Check busy flag of interface 0 is timeout
  * @details  The function is used to send CAN message in BASIC mode of test mode. Before call the API,
  *           the user should be call CAN_EnterTestMode(CAN_TESTR_BASIC) and let CAN controller enter
  *           basic mode of test mode. Please notice IF1 Registers used as Tx Buffer in basic mode.
  */
rt_int32_t CAN_BasicSendMsg(struct c_can_priv * can_priv, struct rt_can_msg* pCanMsg)
{

    UINT32 uRegVal;

    UINT32 i=0;

    while(can_priv->read_reg (can_priv,REG_IF1_CREQ) & CAN_IF_CREQ_BUSY_Msk);//A message transfer is started as soon as the application software has written the message number to
                                                                //the Command Request Register. With this write operation, the Busy bit is automatically set to notify
                                                                //the CPU that a transfer is in progress. After a waiting time of 3 to 6 APB_CLK periods, the transfer
                                                                //between the Interface Register and the Message RAM is completed. The Busy bit is cleared

    uRegVal = can_priv->read_reg (can_priv,REG_STATUS );
    uRegVal &= ~CAN_STATUS_TXOK_Msk;
    can_priv->write_reg (can_priv,(REG_STATUS), uRegVal);

    can_priv->write_reg (can_priv,(REG_IF1_CMASK), CAN_IF_CMASK_WRRD_Msk); //send data

    if (pCanMsg->ide == CAN_STD_ID)
    {
        /* standard ID*/
        can_priv->write_reg (can_priv,(REG_IF1_ARB1), 0x0);
        can_priv->write_reg (can_priv,(REG_IF1_ARB2), (((pCanMsg->id)&0x7FF)<<2));
    }
    else
    {
        /* extended ID*/
        can_priv->write_reg (can_priv,(REG_IF1_ARB1 ), ((pCanMsg->id)&0xFFFF));
        can_priv->write_reg (can_priv,(REG_IF1_ARB2 ), (((pCanMsg->id)&0x1FFF0000)>>16  | CAN_IF_ARB2_XTD_Msk) );
    }

    if(pCanMsg->rtr)  //if it is  remote frame or not
    {
        uRegVal = can_priv->read_reg( can_priv,REG_IF1_ARB2);
        uRegVal |= CAN_IF_ARB2_DIR_Msk;             // direction is  send 
        can_priv->write_reg (can_priv,(REG_IF1_ARB2 ), uRegVal);
    }
    else
    {
        uRegVal = can_priv->read_reg (can_priv,REG_IF1_ARB2);
        uRegVal &= ~CAN_IF_ARB2_DIR_Msk;            //direction is receve 
        can_priv->write_reg (can_priv,(REG_IF1_ARB2), uRegVal);
    }

    can_priv->write_reg (can_priv,REG_IF1_MCON , ((can_priv->read_reg(can_priv,REG_IF1_MCON )) & (~CAN_IF_MCON_DLC_Msk)) | pCanMsg->len );//set  data length
    can_priv->write_reg (can_priv,(REG_IF1_DAT_A1 ), (((rt_uint16_t)pCanMsg->data[1]<<8) | pCanMsg->data[0]));
    can_priv->write_reg (can_priv,(REG_IF1_DAT_A2 ), (((rt_uint16_t)pCanMsg->data[3]<<8) | pCanMsg->data[2]));
    can_priv->write_reg (can_priv,(REG_IF1_DAT_B1 ), (((rt_uint16_t)pCanMsg->data[5]<<8) | pCanMsg->data[4]));
    can_priv->write_reg (can_priv,(REG_IF1_DAT_B2 ), (((rt_uint16_t)pCanMsg->data[7]<<8) | pCanMsg->data[6]));

    /* request transmission*/
    can_priv->write_reg (can_priv,(REG_IF1_CREQ ), (can_priv->read_reg (can_priv,REG_IF1_CREQ ) & (~CAN_IF_CREQ_BUSY_Msk)));//clear busy bit

    if(can_priv->read_reg (can_priv,REG_IF1_CREQ) & CAN_IF_CREQ_BUSY_Msk)
    {
    	 CAN_DBG(" %d %s Cannot clear busy for sending ...\r\n",__LINE__,can_priv->name);//other frame is translatiing
        return FALSE;
    }

    can_priv->write_reg (can_priv,(REG_IF1_CREQ ), (can_priv->read_reg (can_priv,REG_IF1_CREQ) | CAN_IF_CREQ_BUSY_Msk));  // set busy  i will send data now

    for ( i=0; i<0xFFFFF; i++)
    {
        if( (can_priv->read_reg (can_priv,REG_IF1_CREQ) & CAN_IF_CREQ_BUSY_Msk) == 0 ) break; //data send ok
    }

    if ( i >= 0xFFFFF )
    {
    	 CAN_DBG("%d  %s Cannot send out...\r\n",__LINE__,can_priv->name);
        return FALSE;
    }

    return TRUE;
}


/**
  * @brief    Get a message information in BASIC mode.
  *
  * @param[in]    uCAN    CAN channel.
  * @param[out]    pCanMsg     Pointer to the message structure where received data is copied.
  *
  * @return   FALSE  No any message received. \n
  *           TRUE   Receive a message success.
  *
  */
rt_int32_t CAN_BasicReceiveMsg(struct c_can_priv* can_priv,struct rt_can_msg * pCanMsg)
{

    if((can_priv->read_reg (can_priv,REG_IF2_MCON ) & CAN_IF_MCON_NEWDAT_Msk) == 0)  /* In basic mode, receive data always save in IF2 */
    {
        return FALSE;
    }

    can_priv->write_reg (can_priv,(REG_STATUS ), (can_priv->read_reg (can_priv,REG_STATUS ) & (~CAN_STATUS_RXOK_Msk)));

    can_priv->write_reg (can_priv,(REG_IF2_CMASK ), ( CAN_IF_CMASK_ARB_Msk
                                          | CAN_IF_CMASK_CONTROL_Msk
                                          | CAN_IF_CMASK_DATAA_Msk
                                          | CAN_IF_CMASK_DATAB_Msk)); //set where to recve  the frame


    if((can_priv->read_reg (can_priv,REG_IF2_ARB2 ) & CAN_IF_ARB2_XTD_Msk) == 0) // check Message Identifier is standard or extended
    {
        /* standard ID*/
        pCanMsg->ide = CAN_STD_ID;
        pCanMsg->id = (can_priv->read_reg (can_priv,REG_IF2_ARB2 ) >> 2) & 0x07ff;

    }
    else
    {
        /* extended ID*/
        pCanMsg->ide = CAN_EXT_ID;
        pCanMsg->id  = (can_priv->read_reg (can_priv,REG_IF2_ARB2 ) &0x1FFF) << 16;
        pCanMsg->id |= can_priv->read_reg (can_priv,REG_IF2_ARB1 );
    }

    pCanMsg->rtr = !((can_priv->read_reg (can_priv ,REG_IF2_ARB2 ) & CAN_IF_ARB2_DIR_Msk) >> CAN_IF_ARB2_DIR_Pos);

    pCanMsg->len     = can_priv->read_reg (can_priv,REG_IF2_MCON ) & CAN_IF_MCON_DLC_Msk;
    pCanMsg->data[0] = can_priv->read_reg (can_priv,REG_IF2_DAT_A1 ) & CAN_IF_DAT_A1_DATA0_Msk;
    pCanMsg->data[1] = (can_priv->read_reg (can_priv,REG_IF2_DAT_A1 ) & CAN_IF_DAT_A1_DATA1_Msk) >> CAN_IF_DAT_A1_DATA1_Pos;
    pCanMsg->data[2] = can_priv->read_reg (can_priv,REG_IF2_DAT_A2 ) & CAN_IF_DAT_A2_DATA2_Msk;
    pCanMsg->data[3] = (can_priv->read_reg (can_priv,REG_IF2_DAT_A2 ) & CAN_IF_DAT_A2_DATA3_Msk) >> CAN_IF_DAT_A2_DATA3_Pos;
    pCanMsg->data[4] = can_priv->read_reg (can_priv,REG_IF2_DAT_B1 ) & CAN_IF_DAT_B1_DATA4_Msk;
    pCanMsg->data[5] = (can_priv->read_reg (can_priv,REG_IF2_DAT_B1 ) & CAN_IF_DAT_B1_DATA5_Msk) >> CAN_IF_DAT_B1_DATA5_Pos;
    pCanMsg->data[6] = can_priv->read_reg (can_priv,REG_IF2_DAT_B2 ) & CAN_IF_DAT_B2_DATA6_Msk;
    pCanMsg->data[7] = (can_priv->read_reg (can_priv,REG_IF2_DAT_B2 ) & CAN_IF_DAT_B2_DATA7_Msk) >> CAN_IF_DAT_B2_DATA7_Pos;

    return TRUE;
}

/**
  * @brief    Set Rx message object
  * @param[in]    uCAN    CAN channel.
  * @param[in]    u8MsgObj    Specifies the Message object number, from 0 to 31.
  * @param[in]    u8idType    Specifies the identifier type of the frames that will be transmitted
  *                       This parameter can be one of the following values:
  *                       CAN_STD_ID (standard ID, 11-bit)
  *                       CAN_EXT_ID (extended ID, 29-bit)
  * @param[in]    u32id       Specifies the identifier used for acceptance filtering.
  * @param[in]    u8singleOrFifoLast  Specifies the end-of-buffer indicator.
  *                                 This parameter can be one of the following values:
  *                                 TRUE: for a single receive object or a FIFO receive object that is the last one of the FIFO.
  *                                 FALSE: for a FIFO receive object that is not the last one.
  * @retval   TRUE           SUCCESS
  * @retval   FALSE   No useful interface
  * @details  The function is used to configure a receive message object.
  */
rt_int32_t CAN_SetRxMsgObj(struct c_can_priv *can_priv, rt_uint8_t u8MsgObj, rt_uint8_t u8idType, rt_uint8_t rtr, rt_uint32_t u32id, rt_uint8_t u8singleOrFifoLast)
{
    if((can_priv->read_reg (can_priv ,REG_IF2_CREQ) & CAN_IF_CREQ_BUSY_Msk) != 0)
    {
        return FALSE;
    }

    /* Command Setting */
    rt_base_t level= rt_hw_interrupt_disable();
    can_priv->write_reg (can_priv,(REG_IF2_CMASK), (CAN_IF_CMASK_WRRD_Msk | CAN_IF_CMASK_MASK_Msk | CAN_IF_CMASK_ARB_Msk |
                                         CAN_IF_CMASK_CONTROL_Msk | CAN_IF_CMASK_DATAA_Msk | CAN_IF_CMASK_DATAB_Msk));

    if (u8idType == CAN_STD_ID)   /* According STD/EXT ID format,Configure Mask and Arbitration register */
    {
        can_priv->write_reg (can_priv,(REG_IF2_ARB1), 0x0);
        can_priv->write_reg (can_priv,(REG_IF2_ARB2), (CAN_IF_ARB2_MSGVAL_Msk | (u32id & 0x7FF)<< 2));
    }
    else
    {
        can_priv->write_reg (can_priv,(REG_IF2_ARB1), (u32id & 0xFFFF));
        can_priv->write_reg (can_priv,(REG_IF2_ARB2), (CAN_IF_ARB2_MSGVAL_Msk | CAN_IF_ARB2_XTD_Msk | (u32id & 0x1FFF0000)>>16));
    }
    //can_priv->write_reg (can_priv,(REG_IF2_MASK2), (can_priv->read_reg (can_priv,REG_IF2_MASK2) &~ CAN_IF_MASK2_MDIR_Msk ));//ֻмһյңֻ֡յ֡
    can_priv->write_reg (can_priv,(REG_IF2_MCON), (can_priv->read_reg (can_priv,REG_IF2_MCON ) | CAN_IF_MCON_UMASK_Msk | CAN_IF_MCON_RXIE_Msk));//CAN_IF_MCON_INTPND_Msk

    if(u8singleOrFifoLast)
    {
        can_priv->write_reg (can_priv,(REG_IF2_MCON), (can_priv->read_reg (can_priv,REG_IF2_MCON) | CAN_IF_MCON_EOB_Msk));
    }
    else
    {
        can_priv->write_reg (can_priv,(REG_IF2_MCON), (can_priv->read_reg (can_priv,REG_IF2_MCON) & ~CAN_IF_MCON_EOB_Msk));
    }


    if(rtr==REMOTE_FRAME)

    	can_priv->write_reg (can_priv,(REG_IF2_ARB2 ), (can_priv->read_reg (can_priv,REG_IF2_ARB2 ) | CAN_IF_ARB2_DIR_Msk));//DIR=1Է֡  Խң֡
    else
    	can_priv->write_reg (can_priv,(REG_IF2_ARB2 ), (can_priv->read_reg (can_priv,REG_IF2_ARB2 ) & ~CAN_IF_ARB2_DIR_Msk));//DIR=0 Էң֡ Խ֡


    can_priv->write_reg (can_priv,(REG_IF2_DAT_A1), 0x0);
    can_priv->write_reg (can_priv,(REG_IF2_DAT_A2), 0x0);
    can_priv->write_reg (can_priv,(REG_IF2_DAT_B1), 0x0);
    can_priv->write_reg (can_priv,(REG_IF2_DAT_B2), 0x0);

    can_priv->write_reg (can_priv,(REG_IF2_CREQ), (1 + u8MsgObj));
    rt_hw_interrupt_enable(level);
    return TRUE;
}



rt_int32_t CAN_SetMsgObj2(struct c_can_priv *can_priv,rt_uint32_t u8MsgObj, struct rt_can_msg * pCanMsg, rt_uint32_t u32IdMask, rt_uint32_t u32MCON)
	{
	   if((can_priv->read_reg (can_priv ,REG_IF2_CREQ) & CAN_IF_CREQ_BUSY_Msk) != 0)
	    {
	        return FALSE;
	    }

	    /* Command Setting */
	    rt_base_t level= rt_hw_interrupt_disable();
	    can_priv->write_reg (can_priv,(REG_IF2_CMASK), (CAN_IF_CMASK_WRRD_Msk | CAN_IF_CMASK_MASK_Msk | CAN_IF_CMASK_ARB_Msk |
	                                         CAN_IF_CMASK_CONTROL_Msk | CAN_IF_CMASK_DATAA_Msk | CAN_IF_CMASK_DATAB_Msk));

	    if (pCanMsg->ide == CAN_STD_ID)   /* According STD/EXT ID format,Configure Mask and Arbitration register */
	    {
	        can_priv->write_reg (can_priv,(REG_IF2_ARB1), 0x0);
	        can_priv->write_reg (can_priv,(REG_IF2_ARB2), (CAN_IF_ARB2_MSGVAL_Msk | (pCanMsg->id & 0x7FF)<< 2));
	    }
	    else
	    {
	        can_priv->write_reg (can_priv,(REG_IF2_ARB1), (pCanMsg->id  & 0xFFFF));
	        can_priv->write_reg (can_priv,(REG_IF2_ARB2), (CAN_IF_ARB2_MSGVAL_Msk | CAN_IF_ARB2_XTD_Msk | (pCanMsg->id  & 0x1FFF0000)>>16));
	    }

	    if(pCanMsg->rtr==REMOTE_FRAME)

	    	can_priv->write_reg (can_priv,(REG_IF2_ARB2 ), (can_priv->read_reg (can_priv,REG_IF2_ARB2 ) | CAN_IF_ARB2_DIR_Msk));//DIR=1Է֡  Խң֡
	    else
	    	can_priv->write_reg (can_priv,(REG_IF2_ARB2 ), (can_priv->read_reg (can_priv,REG_IF2_ARB2 ) & ~CAN_IF_ARB2_DIR_Msk));//DIR=0 Էң֡ Խ֡


	    can_priv->write_reg (can_priv,(REG_IF2_DAT_A1), 0x0);
	    can_priv->write_reg (can_priv,(REG_IF2_DAT_A2), 0x0);
	    can_priv->write_reg (can_priv,(REG_IF2_DAT_B1), 0x0);
	    can_priv->write_reg (can_priv,(REG_IF2_DAT_B2), 0x0);

	    can_priv->write_reg (can_priv,(REG_IF2_MASK1), u32IdMask);
	    can_priv->write_reg (can_priv,(REG_IF2_MASK2), u32IdMask>>16);
	    can_priv->write_reg (can_priv,(REG_IF2_MCON),pCanMsg->len| u32MCON);

	    can_priv->write_reg (can_priv,(REG_IF2_CREQ), (1 + u8MsgObj));
	    rt_hw_interrupt_enable(level);
	    return TRUE;

	}





/**
  * @brief    Gets the message
  * @param[in]    uCAN    CAN channel.
  * @param[in]    u8MsgObj    Specifies the Message object number, from 0 to 31.
  * @param[in]    u8Release   Specifies the message release indicator.
  *                       This parameter can be one of the following values:
  *                        TRUE: the message object is released when getting the data.
  *                        FALSE:the message object is not released.
  * @param[out]    pCanMsg     Pointer to the message structure where received data is copied.
  * @retval   TRUE   Success
  * @retval   FALSE    No any message received
  * @details  Gets the message, if received.
  */
rt_int32_t CAN_ReadMsgObj(struct c_can_priv * can_priv, rt_uint8_t u8MsgObj, rt_uint8_t u8Release, struct rt_can_msg* pCanMsg)
{

    if (!CAN_IsNewDataReceived(can_priv, u8MsgObj))
    {
        return FALSE;
    }

    can_priv->write_reg (can_priv ,(REG_STATUS), (can_priv->read_reg (can_priv,REG_STATUS) & ~CAN_STATUS_RXOK_Msk));

    /* read the message contents*/
    can_priv->write_reg (can_priv,(REG_IF2_CMASK ), (CAN_IF_CMASK_MASK_Msk
                                         | CAN_IF_CMASK_ARB_Msk
                                         | CAN_IF_CMASK_CONTROL_Msk
                                         | CAN_IF_CMASK_CLRINTPND_Msk
                                         | CAN_IF_CMASK_TXRQSTNEWDAT_Msk 
                                         | CAN_IF_CMASK_DATAA_Msk
                                         | CAN_IF_CMASK_DATAB_Msk
										 |(u8Release ? CAN_IF_CMASK_TXRQSTNEWDAT_Msk :0)));

    can_priv->write_reg (can_priv,(REG_IF2_CREQ), (1 + u8MsgObj));

    while (can_priv->read_reg (can_priv,REG_IF2_CREQ) & CAN_IF_CREQ_BUSY_Msk)
    {
        /*Wait  read opration is completed*/
    }

    if ((can_priv->read_reg (can_priv,REG_IF2_ARB2) & CAN_IF_ARB2_XTD_Msk) == 0)
    {
        /* standard ID*/
        pCanMsg->ide = CAN_STD_ID;
        pCanMsg->id     = (can_priv->read_reg (can_priv,REG_IF2_ARB2) & CAN_IF_ARB2_ID_Msk) >> 2;
    }
    else
    {
        /* extended ID*/
        pCanMsg->ide = CAN_EXT_ID;
        pCanMsg->id  = (((can_priv->read_reg (can_priv,REG_IF2_ARB2)) & 0x1FFF)<<16) | can_priv->read_reg (can_priv,REG_IF2_ARB1);
    }

    pCanMsg->rtr = !((can_priv->read_reg (can_priv,REG_IF2_ARB2) & CAN_IF_ARB2_DIR_Msk) >>CAN_IF_ARB2_DIR_Pos);
    pCanMsg->len    = can_priv->read_reg (can_priv,REG_IF2_MCON) & CAN_IF_MCON_DLC_Msk;
    pCanMsg->data[0] =can_priv->read_reg (can_priv,REG_IF2_DAT_A1) & CAN_IF_DAT_A1_DATA0_Msk;
    pCanMsg->data[1] = (can_priv->read_reg (can_priv,REG_IF2_DAT_A1) & CAN_IF_DAT_A1_DATA1_Msk) >> CAN_IF_DAT_A1_DATA1_Pos;
    pCanMsg->data[2] = can_priv->read_reg (can_priv,REG_IF2_DAT_A2) & CAN_IF_DAT_A2_DATA2_Msk;
    pCanMsg->data[3] = (can_priv->read_reg(can_priv,REG_IF2_DAT_A2) & CAN_IF_DAT_A2_DATA3_Msk) >> CAN_IF_DAT_A2_DATA3_Pos;
    pCanMsg->data[4] = can_priv->read_reg (can_priv,REG_IF2_DAT_B1) & CAN_IF_DAT_B1_DATA4_Msk;
    pCanMsg->data[5] = (can_priv->read_reg (can_priv,REG_IF2_DAT_B1) & CAN_IF_DAT_B1_DATA5_Msk) >> CAN_IF_DAT_B1_DATA5_Pos;
    pCanMsg->data[6] = can_priv->read_reg (can_priv,REG_IF2_DAT_B2) & CAN_IF_DAT_B2_DATA6_Msk;
    pCanMsg->data[7] = (can_priv->read_reg (can_priv,REG_IF2_DAT_B2) & CAN_IF_DAT_B2_DATA7_Msk) >> CAN_IF_DAT_B2_DATA7_Pos;

    return TRUE;
}

static int can_update_spt(int sampl_pt, int tseg, int *tseg1, int *tseg2)
{
    *tseg2 = tseg + 1 - (sampl_pt * (tseg + 1)) / 1000;
    if (*tseg2 < TSEG2_MIN)
        *tseg2 = TSEG2_MIN;
    if (*tseg2 > TSEG2_MAX)
        *tseg2 = TSEG2_MAX;
    *tseg1 = tseg - *tseg2;
    if (*tseg1 > TSEG1_MAX)
    {
        *tseg1 = TSEG1_MAX;
        *tseg2 = tseg - *tseg1;
    }
    return 1000 * (tseg + 1 - *tseg2) / (tseg + 1);
}

/// @endcond HIDDEN_SYMBOLS

/** @addtogroup NUC970_CAN_EXPORTED_FUNCTIONS CAN Exported Functions
  @{
*/


/**
  * @brief    The function is used to disable all CAN interrupt.
  *
  * @param[in]     uCAN    CAN channel.
  *
  * @return   None
  */
void CAN_Close(struct c_can_priv * can_priv)
{
    CAN_DisableInt(can_priv, (CAN_CON_IE_Msk|CAN_CON_SIE_Msk|CAN_CON_EIE_Msk));
}

/**
  * @brief    The function is sets bus timing parameter according current clock and target baud-rate. And set CAN operation mode.
  *
  * @param[in]     uCAN    CAN channel.
  * @param[in]    u32BaudRate The target CAN baud-rate. The range of u32BaudRate is 1~1000KHz
  * @param[in]    u32Mode     The CAN operation mode. ( \ref CAN_NORMAL_MODE / \ref CAN_BASIC_MODE)
  *
  * @return   u32CurrentBitRate  Real baud-rate value
  */
rt_uint32_t CAN_Open(struct c_can_priv * can_priv, rt_uint32_t u32BaudRate, rt_uint32_t u32Mode)
{
    rt_uint32_t u32CurrentBitRate;

    u32CurrentBitRate = CAN_SetBaudRate(can_priv, u32BaudRate);

    if(u32Mode == CAN_BASIC_MODE)
        CAN_EnterTestMode(can_priv, CAN_TEST_BASIC_Msk);

    return u32CurrentBitRate;
}

/**
  * @brief    The function is used to configure a transmit object.
  *
  * @param[in]    uCAN    CAN channel.
  * @param[in]    u32MsgNum   Specifies the Message object number, from 0 to 31
  * @param[in]    pCanMsg     Pointer to the message structure where received data is copied.
  *
  * @return   FALSE: No useful interface. \n
  *           TRUE : Config message object success.
  *
  */
rt_int32_t CAN_SetTxMsg(struct c_can_priv* can_priv,rt_uint32_t u32MsgNum , struct rt_can_msg* pCanMsg)
{
	RT_ASSERT(can_priv !=RT_NULL);
	RT_ASSERT(RT_NULL!=pCanMsg);
    rt_uint32_t i=0;

    while((can_priv->read_reg (can_priv, REG_IF1_CREQ ) & CAN_IF_CREQ_BUSY_Msk) != 0)// this meseage object not be used you can use it ;
    {
        i++;
        if(i > 0x10000000)
            return FALSE;
    }

    /* update the contents needed for transmission*/
    can_priv->write_reg (can_priv,(REG_IF1_CMASK ), 0xF3); /*CAN_CMASK_WRRD_Msk | CAN_CMASK_MASK_Msk | CAN_CMASK_ARB_Msk
                                               | CAN_CMASK_CONTROL_Msk | CAN_CMASK_DATAA_Msk  | CAN_CMASK_DATAB_Msk ; */


    if (pCanMsg->ide == CAN_STD_ID)
    {
        /* standard ID*/
        can_priv->write_reg (can_priv,(REG_IF1_ARB1 ), 0x0);
        can_priv->write_reg (can_priv,(REG_IF1_ARB2 ), ((((pCanMsg->id)&0x7FF)<<2) | CAN_IF_ARB2_DIR_Msk | CAN_IF_ARB2_MSGVAL_Msk));

    }
    else
    {
        /* extended ID*/
        can_priv->write_reg (can_priv,(REG_IF1_ARB1 ), ((pCanMsg->id)&0xFFFF));
        can_priv->write_reg (can_priv,(REG_IF1_ARB2 ), (((pCanMsg->id)&0x1FFF0000)>>16 | CAN_IF_ARB2_DIR_Msk
                                            | CAN_IF_ARB2_XTD_Msk | CAN_IF_ARB2_MSGVAL_Msk));
                                            //the Message Object is configured and should be considered by the Message Handler
    }

    if(pCanMsg->rtr == DATA_FRAME)
        can_priv->write_reg (can_priv,(REG_IF1_ARB2 ), (can_priv->read_reg (can_priv,REG_IF1_ARB2 ) | CAN_IF_ARB2_DIR_Msk));//DIR=1Է֡  Խң֡
    else
        can_priv->write_reg (can_priv,(REG_IF1_ARB2 ), (can_priv->read_reg (can_priv,REG_IF1_ARB2 ) & ~CAN_IF_ARB2_DIR_Msk));//DIR=0 Էң֡Խ֡

    can_priv->write_reg (can_priv,(REG_IF1_DAT_A1 ), (((rt_uint16_t)(pCanMsg->data[1]<<8)) | pCanMsg->data[0]));
    can_priv->write_reg (can_priv,(REG_IF1_DAT_A2 ), (((rt_uint16_t)pCanMsg->data[3]<<8) | pCanMsg->data[2]));
    can_priv->write_reg (can_priv,(REG_IF1_DAT_B1 ), (((rt_uint16_t)pCanMsg->data[5]<<8) | pCanMsg->data[4]));
    can_priv->write_reg (can_priv,(REG_IF1_DAT_B2 ), (((rt_uint16_t)pCanMsg->data[7]<<8) | pCanMsg->data[6]));

    can_priv->write_reg (can_priv,(REG_IF1_MCON ), (CAN_IF_MCON_NEWDAT_Msk | pCanMsg->len | CAN_IF_MCON_EOB_Msk|CAN_IF_MCON_TXRQST_Msk)); // intpend ֮ǽж CAN_IF_MCON_INTPND_Msk  //|CAN_IF_MCON_TXIE_Msk
    can_priv->write_reg (can_priv,(REG_IF1_CREQ ), (1 + u32MsgNum));

    return TRUE;
}

/**
  * @brief    Set transmit request bit
  *
  * @param[in]    uCAN    CAN channel.
  * @param[in]    u32MsgNum    Specifies the Message object number, from 0 to 31.
  *
  * @return   TRUE: Start transmit message.
  */
rt_int32_t CAN_TriggerTxMsg(struct c_can_priv * can_priv, rt_uint32_t u32MsgNum)
{
	RT_ASSERT(can_priv !=RT_NULL);
    can_priv->write_reg (can_priv,(REG_IF1_CMASK ), (CAN_IF_CMASK_WRRD_Msk |CAN_IF_CMASK_TXRQSTNEWDAT_Msk));
    can_priv->write_reg (can_priv,(REG_IF1_CREQ ), (1 + u32MsgNum));

    return TRUE;
}

/**
  * @brief    Enable CAN interrupt
  *
  * @param[in]    uCAN    CAN channel.
  * @param[in]    u32Mask    Interrupt Mask. ( \ref CAN_CON_IE_Msk / \ref CAN_CON_SIE_Msk / \ref CAN_CON_EIE_Msk)
  *
  * @return   None
  */
void CAN_EnableInt(struct c_can_priv * can_priv, rt_uint32_t u32Mask)
{
	  RT_ASSERT(can_priv !=RT_NULL);
	  RT_ASSERT(u32Mask !=RT_NULL);

//    CAN_EnterInitMode(can_priv);

    can_priv->write_reg (can_priv,(REG_CON ), ((can_priv->read_reg (can_priv,REG_CON ) & 0xF1) | ((u32Mask & CAN_CON_IE_Msk   )? CAN_CON_IE_Msk :0)
                                   | ((u32Mask & CAN_CON_SIE_Msk  )? CAN_CON_SIE_Msk:0)
                                   | ((u32Mask & CAN_CON_EIE_Msk  )? CAN_CON_EIE_Msk:0)));

 //   CAN_LeaveInitMode(can_priv);
}

/**
  * @brief    Disable CAN interrupt
  *
  * @param[in]    uCAN    CAN channel.
  * @param[in]    u32Mask    Interrupt Mask. ( \ref CAN_CON_IE_Msk / \ref CAN_CON_SIE_Msk / \ref CAN_CON_EIE_Msk)
  *
  * @return   None
  */

void CAN_DisableInt(struct c_can_priv * can_priv,rt_uint32_t u32Mask)
{

    CAN_EnterInitMode(can_priv);

    can_priv->write_reg (can_priv,(REG_CON ), (can_priv->read_reg (can_priv,REG_CON ) & ~(CAN_CON_IE_Msk | ((u32Mask & CAN_CON_SIE_Msk)?CAN_CON_SIE_Msk:0)
                                   | ((u32Mask & CAN_CON_EIE_Msk)?CAN_CON_EIE_Msk:0))));

    CAN_LeaveInitMode(can_priv);
}


/**
  * @brief    The function is used to configure a receive message object
  *
  * @param[in]    uCAN    CAN channel.
  * @param[in]    u32MsgNum   Specifies the Message object number, from 0 to 31
  * @param[in]    u32IDType   Specifies the identifier type of the frames that will be transmitted. ( \ref CAN_STD_ID / \ref CAN_EXT_ID)
  * @param[in]    u32ID       Specifies the identifier used for acceptance filtering.
  *
  * @return   FALSE: No useful interface \n
  *           TRUE : Configure a receive message object success.
  *
  */
rt_int32_t CAN_SetRxMsg(struct c_can_priv * can_priv , rt_uint32_t u32MsgNum , rt_uint32_t u32IDType, rt_uint8_t rtr,rt_uint32_t u32ID)
{
	RT_ASSERT(can_priv!=RT_NULL);
    rt_uint32_t u32TimeOutCount = 0;

    while(CAN_SetRxMsgObj(can_priv, u32MsgNum, u32IDType,rtr, u32ID, TRUE) == FALSE)
    {
        u32TimeOutCount++;

        if(u32TimeOutCount >= 0x10000000) return FALSE;
    }

    return TRUE;
}

rt_int32_t CAN_SetRxMsg2(struct c_can_priv * can_priv , rt_uint32_t u32MsgNum ,struct rt_can_msg * pCanMsg, rt_uint32_t u32IdMask, rt_uint32_t u32MCON)
{
	RT_ASSERT(can_priv!=RT_NULL);
    rt_uint32_t u32TimeOutCount = 0;

    while(CAN_SetMsgObj2(can_priv,u32MsgNum, pCanMsg, u32IdMask,  u32MCON) == FALSE)
    {
        u32TimeOutCount++;

        if(u32TimeOutCount >= 0x10000000) return FALSE;
    }

    return TRUE;
}

/**
  * @brief    The function is used to configure several receive message objects
  *
  * @param[in]    uCAN    CAN channel.
  * @param[in]    u32MsgNum   The starting MSG RAM number. (0 ~ 31)
  * @param[in]    u32MsgCount the number of MSG RAM of the FIFO.
  * @param[in]    u32IDType   Specifies the identifier type of the frames that will be transmitted. ( \ref CAN_STD_ID / \ref CAN_EXT_ID)
  * @param[in]    u32ID       Specifies the identifier used for acceptance filtering.
  *
  * @return   FALSE: No useful interface \n
  *           TRUE : Configure receive message objects success.
  *
  */
rt_int32_t CAN_SetMultiRxMsg(struct c_can_priv * can_priv, rt_uint32_t u32MsgNum , rt_uint32_t u32MsgCount, rt_uint32_t u32IDType,rt_uint8_t rtr, rt_uint32_t u32ID)
{
    rt_uint32_t i = 0;
    rt_uint32_t u32TimeOutCount;
    rt_uint32_t u32EOB_Flag = 0;

    for(i= 1; i < u32MsgCount; i++)
    {
        u32TimeOutCount = 0;

        u32MsgNum += (i - 1);

        if(i == u32MsgCount) u32EOB_Flag = 1;

        while(CAN_SetRxMsgObj(can_priv, u32MsgNum, u32IDType, rtr,u32ID, u32EOB_Flag) == FALSE)
        {
            u32TimeOutCount++;

            if(u32TimeOutCount >= 0x10000000) return FALSE;
        }
    }

    return TRUE;
}


/**
  * @brief    Send CAN message.
  * @param[in]    uCAN    CAN channel.
  * @param[in]    u32MsgNum   Specifies the Message object number, from 0 to 31
  * @param[in]    pCanMsg     Pointer to the message structure where received data is copied.
  *
  * @return   FALSE: When operation in basic mode: Transmit message time out, or when operation in normal mode: No useful interface. \n
  *           TRUE : Transmit Message success.
  */
rt_err_t CAN_Transmit(struct c_can_priv * can_priv , rt_uint32_t u32MsgNum ,struct  rt_can_msg* pCanMsg)
{
	RT_ASSERT(can_priv!=RT_NULL);
	RT_ASSERT(pCanMsg!=RT_NULL);
	rt_uint16_t status =0;
    if((can_priv->read_reg (can_priv,REG_CON) & CAN_CON_TEST_Msk) && (can_priv->read_reg (can_priv,REG_TEST) & CAN_TEST_BASIC_Msk))
    {
        return (CAN_BasicSendMsg(can_priv, pCanMsg));
    }
    else
    {

        if(CAN_SetTxMsg(can_priv, u32MsgNum, pCanMsg) == FALSE)
            return RT_ERROR;//FALSE;
        CAN_TriggerTxMsg(can_priv, u32MsgNum);
    }

 //   status =can_priv->read_reg(can_priv,REG_STATUS);
 //   if(status & CAN_STATUS_TXOK_Msk )  rt_kprintf("tx ok\r\n");
	 CAN_IsTxRqstPending(can_priv, u32MsgNum);


    return RT_EOK;
}


/**
  * @brief    Gets the message, if received.
  * @param[in]    uCAN    CAN channel.
  * @param[in]    u32MsgNum   Specifies the Message object number, from 0 to 31
  * @param[out]    pCanMsg     Pointer to the message structure where received data is copied.
  *
  * @return   FALSE: No any message received. \n
  *           TRUE : Receive Message success.
  */

rt_int32_t CAN_Receive(struct c_can_priv * can_priv, rt_uint32_t u32MsgNum , struct rt_can_msg* pCanMsg)
{
	RT_ASSERT(can_priv!=RT_NULL);
	RT_ASSERT(pCanMsg!=RT_NULL);
    if((can_priv->read_reg (can_priv,REG_CON ) & CAN_CON_TEST_Msk) && (can_priv->read_reg (can_priv,REG_TEST ) & CAN_TEST_BASIC_Msk))
    {
        return (CAN_BasicReceiveMsg(can_priv, pCanMsg));
    }
    else
    {
        return CAN_ReadMsgObj(can_priv, u32MsgNum, TRUE, pCanMsg);
    }
}

/**
  * @brief    Clear interrupt pending bit.
  * @param[in]    uCAN    CAN channel.
  * @param[in]    u32MsgNum   Specifies the Message object number, from 0 to 31
  *
  * @return   None
  *
  */
void CAN_CLR_INT_PENDING_BIT(struct c_can_priv * can_priv,rt_uint8_t u32MsgNum)
{
	RT_ASSERT(can_priv !=RT_NULL);
    rt_uint32_t u32MsgIfNum = 0;
    rt_uint32_t u32IFBusyCount = 0;

    while(u32IFBusyCount < 0x10000000)
    {
        if((can_priv->read_reg (can_priv,REG_IF1_CREQ ) & CAN_IF_CREQ_BUSY_Msk) == 0)
        {
            u32MsgIfNum = 0;
            break;
        }
        else if((can_priv->read_reg (can_priv,REG_IF2_CREQ )  & CAN_IF_CREQ_BUSY_Msk) == 0)
        {
            u32MsgIfNum = 1;
            break;
        }

        u32IFBusyCount++;
    }

    if(u32MsgIfNum == 0)
    {
        can_priv->write_reg (can_priv,(REG_IF1_CMASK ), (CAN_IF_CMASK_CLRINTPND_Msk | CAN_IF_CMASK_TXRQSTNEWDAT_Msk));
        can_priv->write_reg (can_priv,(REG_IF1_CREQ ), (1 + u32MsgNum));
    }
    else if(u32MsgIfNum == 1)
    {
        can_priv->write_reg (can_priv,REG_IF2_CMASK , (CAN_IF_CMASK_CLRINTPND_Msk | CAN_IF_CMASK_TXRQSTNEWDAT_Msk));
        can_priv->write_reg (can_priv,(REG_IF2_CREQ ), (1 + u32MsgNum));
    }
}


/*@}*/ /* end of group NUC970_CAN_EXPORTED_FUNCTIONS */

/*@}*/ /* end of group NUC970_CAN_Driver */

/*@}*/ /* end of group NUC970_Device_Driver */


/*** (C) COPYRIGHT 2015 Nuvoton Technology Corp. ***/



/******************************************rt_thread interface*********************************/


static rt_uint16_t c_can_plat_read_reg_aligned_to_32bit(struct c_can_priv *priv,enum reg2 index)
{

	return inp16((rt_uint32_t)priv->base + 2 * priv->regs[index]);
}

static void c_can_plat_write_reg_aligned_to_32bit(struct c_can_priv *priv,enum reg2 index, rt_uint16_t val)
{
//	rt_uint32_t base1=priv->base;
//	rt_uint32_t  base2 = 2* priv->regs[index] ;
//	rt_uint32_t base3=base1+base2;
//	rt_uint32_t base4 =(rt_uint32_t)priv->base + (2 * priv->regs[index]);
//	outp16(base3,val);
	outp16( (rt_uint32_t)priv->base + 2 * priv->regs[index] ,val);
}

/**
  * @brief    The function is used to set bus timing parameter according current clock and target baud-rate.
  *
  * @param[in]    uCAN    CAN channel.
  * @param[in]    u32BaudRate The target CAN baud-rate. The range of u32BaudRate is 1~1000KHz
  *
  * @return   u32CurrentBitRate  Real baud-rate value
  */
rt_uint32_t CAN_SetBaudRate(struct c_can_priv *can_priv, rt_uint32_t u32BaudRate)
{


    long rate;
    long best_error = 1000000000, error = 0;
    int best_tseg = 0, best_brp = 0, brp = 0;
    int tsegall, tseg = 0, tseg1 = 0, tseg2 = 0;
    int spt_error = 1000, spt = 0, sampl_pt;
    uint64_t clock_freq = 0;
    rt_uint32_t sjw = 1;
    rt_uint16_t bittimer=0;

    CAN_EnterInitMode(can_priv);

    clock_freq = sysGetClock(SYS_PCLK);
    clock_freq = clock_freq*1000000;

    if(u32BaudRate >= 1000000)
        u32BaudRate = 1000000;

    /* Use CIA recommended sample points */
    if (u32BaudRate > 800000)
        sampl_pt = 750;
    else if (u32BaudRate > 500000)
        sampl_pt = 800;
    else
        sampl_pt = 875;

    /* tseg even = round down, odd = round up */
    for (tseg = (TSEG1_MAX + TSEG2_MAX) * 2 + 1; tseg >= (TSEG1_MIN + TSEG2_MIN) * 2; tseg--)
    {
        tsegall = 1 + tseg / 2;
        /* Compute all possible tseg choices (tseg=tseg1+tseg2) */
        brp = clock_freq / (tsegall * u32BaudRate) + tseg % 2;
        /* chose brp step which is possible in system */
        brp = (brp / BRP_INC) * BRP_INC;

        if ((brp < BRP_MIN) || (brp > BRP_MAX))
            continue;
        rate = clock_freq / (brp * tsegall);

        error = u32BaudRate - rate;

        /* tseg brp biterror */
        if (error < 0)
            error = -error;
        if (error > best_error)
            continue;
        best_error = error;
        if (error == 0)
        {
            spt = can_update_spt(sampl_pt, tseg / 2, &tseg1, &tseg2);
            error = sampl_pt - spt;
            if (error < 0)
                error = -error;
            if (error > spt_error)
                continue;
            spt_error = error;
        }
        best_tseg = tseg / 2;
        best_brp = brp;

        if (error == 0)
            break;
    }

    spt = can_update_spt(sampl_pt, best_tseg, &tseg1, &tseg2);

    /* check for sjw user settings */
    /* bt->sjw is at least 1 -> sanitize upper bound to sjw_max */
    if (sjw > SJW_MAX)
        sjw = SJW_MAX;
    /* bt->sjw must not be higher than tseg2 */
    if (tseg2 < sjw)
        sjw = tseg2;

    /* real bit-rate */
    u32BaudRate = clock_freq / (best_brp * (tseg1 + tseg2 + 1));

    can_priv->write_reg(can_priv,REG_BTIME, (((rt_uint16_t)(tseg2 - 1) << CAN_BTIME_TSEG2_Pos) | ((rt_uint32_t)(tseg1 - 1) << CAN_BTIME_TSEG1_Pos)|((best_brp - 1) & CAN_BTIME_BRP_Msk) | (sjw << CAN_BTIME_SJW_Pos)));
    can_priv->write_reg(can_priv,REG_BRPE, (((best_brp - 1) >> 6) & 0x0F));

    rt_kprintf("\n bitrate = %d \n", CAN_GetCANBitRate(can_priv));

    CAN_LeaveInitMode(can_priv);

    return u32BaudRate;
}

/**
  * @brief    The function is used to  config  data filter  baud-rate work mode .
  *
  * @param[in]    dev_can    CAN channel.
  * @param[in]    cfg  parmters need to set
  *  @return   	  set ok or not
  */
static rt_err_t drv_configure(struct rt_can_device *dev_can,
                              struct can_configure *cfg)
{
	rt_uint16_t mode_type=0,mode_type1=0;
    RT_ASSERT(dev_can);
    RT_ASSERT(cfg);

    struct  c_can_priv * can_priv = (struct c_can_priv *)dev_can->parent.user_data;

    switch (cfg->mode)
    {
    case RT_CAN_MODE_NORMAL:
    	// CAN_EnterInitMode(can_priv);
    //    mode_type = CAN_MODE_NORMAL;
   //     can_priv->write_reg(can_priv,REG_CON,can_priv->read_reg(can_priv,REG_CON)&~mode_type);
      //  CAN_LeaveInitMode(can_priv);
        break;
    case RT_CAN_MODE_LISEN:
    	CAN_EnterInitMode(can_priv);
    	can_priv->write_reg(can_priv,REG_CON,can_priv->read_reg(can_priv,REG_CON)|mode_type);
    	 mode_type1 = CAN_MODE_SILENT;
    	 can_priv->write_reg(can_priv,REG_TEST,can_priv->read_reg(can_priv,REG_TEST)|mode_type1);
    	 CAN_LeaveInitMode(can_priv);
        break;
    case RT_CAN_MODE_LOOPBACK:
    	 CAN_EnterInitMode(can_priv);
    	can_priv->write_reg(can_priv,REG_CON,can_priv->read_reg(can_priv,REG_CON)|mode_type);
    	 mode_type1 = CAN_MODE_LOOPBACK;
    	 can_priv->write_reg(can_priv,REG_TEST,can_priv->read_reg(can_priv,REG_TEST)|mode_type1);
    	 CAN_LeaveInitMode(can_priv);
        break;
    case RT_CAN_MODE_LOOPBACKANLISEN:
    	 CAN_EnterInitMode(can_priv);
    	can_priv->write_reg(can_priv,REG_CON,can_priv->read_reg(can_priv,REG_CON)|mode_type);
    	mode_type1 = CAN_MODE_SILENT_LOOPBACK;
        can_priv->write_reg(can_priv,REG_TEST,can_priv->read_reg(can_priv,REG_TEST)|mode_type1);
        CAN_LeaveInitMode(can_priv);
        break;
    }



   // can_priv->write_reg(can_priv,REG_TEST,can_priv->read_reg(can_priv,REG_TEST)|mode_type1);
        		// CAN0
    		// CAN0
	outpw(REG_SYS_GPH_MFPL, (inpw(REG_SYS_GPH_MFPL) & 0xffff00ff) | 0xCC00 ); // GPH_2,GPH_3 // RX, TX

	//CAN_1
	outpw(REG_SYS_GPH_MFPH, (inpw(REG_SYS_GPH_MFPH) & 0x00ffffff) | 0xCC000000 ); // GPH_14,GPH_15 // RX, TX

	outpw(REG_CLK_PCLKEN1, (inpw(REG_CLK_PCLKEN1) | (1 << 8) | (1 << 9)) );

    CAN_SetBaudRate(can_priv, cfg->baud_rate);

    /* Filter conf */
    //TODO:
	//CAN_SetRxMsg(UINT32 uCAN, uint32_t u32MsgNum , uint32_t u32IDType, uint32_t u32ID)
	//CAN_SetRxMsgObj(can_priv,0,CAN_EXT_ID, 0x12345, 1); // initialzation  oene  meseag object;
    return RT_EOK;
}

 void nuc97x_can_isr(int vector,void *param);

static rt_err_t drv_control(struct rt_can_device *can, int cmd, void *arg)
{
   // struct stm32_drv_can *drv_can;
    struct c_can_priv * can_priv ;
    rt_uint32_t argval;
    struct  filter_config2 * msg_set;

    rt_uint16_t intstate;

   // drv_can = (struct stm32_drv_can *) can->parent.user_data;
    can_priv =(struct c_can_priv *) can->parent.user_data;
    //assert_param(drv_can != RT_NULL);
    assert_param(can_priv != RT_NULL);
    switch (cmd)
    {
    case RT_DEVICE_CTRL_CLR_INT:
        argval = (rt_uint32_t) arg;
        if (argval == RT_DEVICE_CAN_INT_ERR)
        {
           CAN_DisableInt(can_priv, CAN_CON_EIE_Msk);
        }
        else if (argval == RT_DEVICE_CAN_INT_SIE)
        {
        CAN_DisableInt(can_priv, CAN_CON_SIE_Msk);
        }
        else if (argval == RT_DEVICE_CAN_INT_ALL)
        {
        CAN_DisableInt(can_priv, (CAN_CON_IE_Msk|CAN_CON_SIE_Msk|CAN_CON_EIE_Msk));
        }
        break;
    case RT_DEVICE_CTRL_SET_INT:
    	 argval = (rt_uint32_t) arg;
         rt_hw_interrupt_install(can_priv->irqno, nuc97x_can_isr, can,can_priv->name);
         /* Set the irq and irq trigger level for can */
         rt_hw_interrupt_set_priority(can_priv->irqno, IRQ_LEVEL_1);
      // rt_hw_interrupt_set_priority(can_priv->irqno, IRQ_LEVEL_1);
      //  rt_hw_interrupt_set_type(can_priv->irqno,HIGH_LEVEL_SENSITIVE);
       // SysEnableInterrupt(ENABLE_IRQ);
        rt_hw_interrupt_umask(can_priv->irqno);
        if (argval == RT_DEVICE_FLAG_INT_RX)
        {

         CAN_EnableInt(can_priv, (CAN_CON_IE_Msk|CAN_CON_SIE_Msk|CAN_CON_EIE_Msk));
         //TODO set rx_ind
        // can_priv->write_reg(can_priv,REG_IF2_MCON,  CAN_IF_MCON_TXIE_Msk|1ul << CAN_IF_MCON_RXIE_Pos);
        }
        else if (argval == RT_DEVICE_FLAG_INT_TX)
        {
         CAN_EnableInt(can_priv, (CAN_CON_IE_Msk|CAN_CON_SIE_Msk|CAN_CON_EIE_Msk));
        }
        else if (argval == RT_DEVICE_CAN_INT_ERR )
        {
        	 CAN_EnableInt(can_priv, (CAN_CON_IE_Msk|CAN_CON_SIE_Msk|CAN_CON_EIE_Msk));
        }

        break;
    case RT_CAN_CMD_SET_FILTER:
        /* TODO: filter*/

    	 msg_set = (struct filter_config2 *)arg;
    	 //CAN_SetRxMsg(....);
        CAN_SetRxMsg2(can_priv,msg_set->msgobj,msg_set->msg,msg_set->u32mask,msg_set->u32mcon); // initialzation  oene  meseag object;
        break;
    case RT_CAN_CMD_SET_MODE:
        argval = (rt_uint32_t) arg;
        if (argval != RT_CAN_MODE_NORMAL ||
            argval != RT_CAN_MODE_LISEN ||
            argval != RT_CAN_MODE_LOOPBACK ||
            argval != RT_CAN_MODE_LOOPBACKANLISEN)
        {
            return RT_ERROR;
        }
        if (argval != can->config.mode)
        {
            CAN_EnterInitMode(can_priv);
            can->config.mode = argval;
            if(argval & RT_CAN_MODE_LOOPBACKANLISEN)
            {
                CAN_EnterTestMode( can_priv, CAN_MODE_SILENT_LOOPBACK);
            }
            else if ( argval & RT_CAN_MODE_LOOPBACK)
            {
                CAN_EnterTestMode( can_priv, CAN_MODE_LOOPBACK);
            }
            else if (argval & RT_CAN_MODE_LISEN)
            {
                CAN_EnterTestMode( can_priv,  CAN_MODE_SILENT);
            }
            else if (argval & RT_CAN_MODE_NORMAL)
            {

                can_priv->write_reg (can_priv,(REG_CON ), can_priv->read_reg(can_priv,REG_CON)&~(1<<7));

            }
            can_priv->write_reg (can_priv,(REG_CON ), can_priv->read_reg(can_priv,REG_CON|CAN_CON_DAR_Msk));//disable automatic retransmission;
            CAN_LeaveInitMode(can_priv);

        }
        break;
    case RT_CAN_CMD_SET_BAUD:
    	argval=(rt_uint32_t)arg;
         CAN_SetBaudRate(can_priv, argval);
        break;
    case RT_CAN_CMD_SET_PRIV:
        argval = (rt_uint32_t) arg;
         break;
    case RT_CAN_CMD_GET_STATUS:
    {
        rt_uint16_t errtype;
        errtype = can_priv->read_reg (can_priv, REG_ERR);

        can->status.rcverrcnt = errtype >> 8;
        can->status.snderrcnt = (errtype  & 0x00FF);
        errtype = can_priv->read_reg(can_priv, REG_STATUS);
        can->status.errcode = errtype & 0x07;
        if (arg != &can->status)
        {
            rt_memcpy(arg, &can->status, sizeof(can->status));
        }
    }
    break;
    }

    return RT_EOK;
}


 int drv_recvmsg(struct rt_can_device *can, void *buf, rt_uint32_t boxno)
{
    struct c_can_priv * can_priv = (struct c_can_priv *) can->parent.user_data;
    if( CAN_ReadMsgObj(can_priv, boxno,1, buf)==FALSE)
    	return -1;
    else
    	return RT_EOK;
}


static int drv_sendmsg(struct rt_can_device *can, const void *buf, rt_uint32_t boxno)
{
    struct c_can_priv * can_priv =(struct c_can_priv *)can->parent.user_data;
    return CAN_Transmit(can_priv, boxno,buf);
}



static const struct rt_can_ops drv_can_ops =
{
    drv_configure,
    drv_control,
    drv_sendmsg,
    drv_recvmsg,
};

#ifdef RT_CAN_USING_CAN1
static struct c_can_priv  drv_can1;
struct rt_can_device dev_can1;
#endif
#ifdef RT_CAN_USING_CAN2
static struct c_can_priv  drv_can2;
struct rt_can_device dev_can2;
#endif

int rt_hw_can_init(void)
{
    struct c_can_priv * drv_can;
    struct can_configure config = CANDEFAULTCONFIG;
    config.privmode = 0;
    config.ticks = 50;
    config.sndboxnumber = 2;//3
    config.msgboxsz = 32;
#ifdef RT_CAN_USING_HDR
    config.maxhdr = 14;

#endif

#ifdef RT_CAN_USING_CAN1
    drv_can = &drv_can1;
    drv_can->base = CAN0_BA;  //can 1 register base adress;
    drv_can->read_reg = c_can_plat_read_reg_aligned_to_32bit;
    drv_can->write_reg = c_can_plat_write_reg_aligned_to_32bit;
    drv_can->regs= reg_map_c_can;
    drv_can->irqno=CAN0_IRQn;
    drv_can->name = "CAN1";
    outpw(REG_SYS_GPH_MFPL, (inpw(REG_SYS_GPH_MFPL) & 0xffff00ff) | 0xCC00 ); // GPH_2,GPH_3 // RX, TX pin config


    dev_can1.ops    = &drv_can_ops;
    dev_can1.config = config;
    /* register CAN1 device */
    rt_hw_can_register(&dev_can1, "can1",
                       &drv_can_ops,
                       drv_can);
#endif /* BSP_USING_CAN1 */

#ifdef RT_CAN_USING_CAN2
    drv_can = &drv_can2;
    drv_can->base =CAN1_BA;  //can 1 register base adress;
    drv_can->read_reg = c_can_plat_read_reg_aligned_to_32bit;
    drv_can->write_reg = c_can_plat_write_reg_aligned_to_32bit;
    drv_can->regs= reg_map_c_can;
    drv_can->irqno=CAN1_IRQn;
    drv_can->name = "CAN2";
    outpw(REG_SYS_GPH_MFPH, (inpw(REG_SYS_GPH_MFPH) & 0x00ffffff) | 0xCC000000 ); // GPH_14,GPH_15 // RX, TX pin config

    dev_can2.config = config;
    /* register CAN2 device */
    rt_hw_can_register(&dev_can2, "can2",
                       &drv_can_ops,
                       drv_can);
#endif /* BSP_USING_CAN2 */
    outpw(REG_CLK_PCLKEN1, (inpw(REG_CLK_PCLKEN1) | (1 << 8) | (1 << 9)) );//  can1 can2  clk enable;
    return 0;
}

INIT_DEVICE_EXPORT( rt_hw_can_init);



/*----------------------------------------------------------------------------*/
/*                    ���ڴ�ӡCAN���ݰ����ݣ������ã�                          */
/*----------------------------------------------------------------------------*/
void CAN_ShowMsg(struct rt_can_msg* Msg)
{
    RT_ASSERT(Msg!=RT_NULL);
    rt_uint8_t i;
    rt_kprintf(" ID=%8X, Type=%s, DLC=%d,Data=",Msg->id,Msg->ide?"EXT":"STD",Msg->len);
    for(i=0; i<Msg->len; i++)
    rt_kprintf("%02X,",Msg->data[i]);
    rt_kprintf("\r\n\r\n");
}





/* CAN �жϽ��մ�������
void CAN_MsgInterrupt(struct c_can_priv can_priv, rt_uint32_t u32IIDR)
{
	CAN_DEBUG("Msg-%d �жϻص�. \r\n", (u32IIDR-1));
    CAN_Receive(can_priv, (u32IIDR-1),&rrMsg);
    CAN_ShowMsg(&rrMsg);
}
*/



// CAN中断服务函数
 void nuc97x_can_isr(int vector,void *param)
{
	rt_base_t level =  rt_hw_interrupt_disable();
    rt_uint32_t u8IIDRstatus;
    rt_uint32_t new_data;
    RT_ASSERT(param != RT_NULL);
    struct rt_can_device *can = (struct rt_can_device *)param;
    struct c_can_priv* can_priv = (struct c_can_priv *)can->parent.user_data;
    CanRxMsgTypeDef*  pCanMsg =&can_priv->rx_message;
    u8IIDRstatus = can_priv->read_reg(can_priv ,REG_IIDR);

    if(u8IIDRstatus == 0x00008000) {      /* Check Status Interrupt Flag (Error status Int and Status change Int) */
        /**************************/
        /* Status Change interrupt*/
        /**************************/
    	 new_data = can_priv->read_reg (can_priv,REG_STATUS );
        if(can_priv->read_reg(can_priv,REG_STATUS) & CAN_STATUS_RXOK_Msk) {
            can_priv->write_reg(can_priv,REG_STATUS, can_priv->read_reg(can_priv,REG_STATUS) & ~CAN_STATUS_RXOK_Msk);  /* Clear Rx Ok status*/

            CAN_DBG("%d  %s RX OK INT\r\n", __LINE__,can_priv->name) ;
/*
            new_data = can_priv->read_reg (can_priv,REG_ERR );
            new_data=can_priv->read_reg (can_priv,REG_MVLD2 )<<16 |can_priv->read_reg (can_priv,REG_MVLD1 );
         new_data=can_priv->read_reg (can_priv,REG_TXREQ2 )<<16 |can_priv->read_reg (can_priv,REG_TXREQ1 );
   		 new_data=can_priv->read_reg (can_priv,REG_NDAT2 )<<16 |can_priv->read_reg (can_priv,REG_NDAT1 );
   		 new_data = can_priv->read_reg(can_priv ,REG_IIDR);
  		 new_data=can_priv->read_reg (can_priv,REG_NDAT2 )<<16 |can_priv->read_reg (can_priv,REG_NDAT1 );
  		 new_data = can_priv->read_reg(can_priv ,REG_IIDR);
  		 new_data=can_priv->read_reg (can_priv,REG_IPND2 )<<16 |can_priv->read_reg (can_priv,REG_IPND1 );
  		 new_data=can_priv->read_reg (can_priv,REG_IPND2 )<<16 |can_priv->read_reg (can_priv,REG_IPND1 );
  		 */
        }

        if(can_priv->read_reg(can_priv,REG_STATUS) & CAN_STATUS_TXOK_Msk) {
            can_priv->write_reg(can_priv,REG_STATUS, can_priv->read_reg(can_priv,REG_STATUS) & ~CAN_STATUS_TXOK_Msk);  /* Clear Tx Ok status*/

            CAN_DBG("%d  %s TX OK INT\r\n",__LINE__,can_priv->name) ;
            struct rt_can_tx_fifo *tx_fifo;
            struct rt_can_sndbxinx_list *tx_tosnd = RT_NULL;
			tx_fifo= (struct rt_can_tx_fifo *) can->can_tx;
		    RT_ASSERT(tx_fifo != RT_NULL);
		    CAN_DBG("%d  send channel number %d \r\n",__LINE__,tx_fifo->send_done_num) ;
            rt_hw_can_isr(can, RT_CAN_EVENT_TX_DONE | tx_fifo->send_done_num << 8);
        }

        /**************************/
        /* Error Status interrupt */
        /**************************/
        if(can_priv->read_reg(can_priv,REG_STATUS) & CAN_STATUS_EWARN_Msk) {
        	 CAN_DBG("%d  %s EWARN INT\r\n",__LINE__,can_priv->name) ;
        }

        if(can_priv->read_reg(can_priv,REG_STATUS) & CAN_STATUS_BOFF_Msk) {
        	 CAN_DBG("%d  %s BOFF INT\r\n",__LINE__,can_priv->name) ;
        }
    } else if (u8IIDRstatus!=0) {
    	 CAN_DBG("%d  %s recv data in  %d channel\r\n",__LINE__,can_priv->name, (u8IIDRstatus-1));


		 new_data=can_priv->read_reg (can_priv,REG_NDAT2 )<<16 |can_priv->read_reg (can_priv,REG_NDAT1 );

        /* save to user fifo */
        rt_hw_can_isr(can, RT_CAN_EVENT_RX_IND | (u8IIDRstatus-1) << 8);

        CAN_CLR_INT_PENDING_BIT(can_priv, (u8IIDRstatus-1));      /* Clear Interrupt Pending */

    }else if(can_priv->read_reg(can_priv,REG_WU_STATUS) == 1) {
    	 CAN_DBG("%d %s Wake up\r\n",__LINE__,can_priv->name);

        can_priv->write_reg(can_priv,REG_WU_STATUS, 0x0);  /* Write '0' to clear */
    }
    rt_hw_interrupt_enable(level);

}
